package net.knarcraft.blacksmith.config;

/**
 * An enum representing all of Blacksmith's settings
 */
public enum NPCSetting {

    /**
     * The setting for whether the NPC should drop an item to the ground when finished
     *
     * <p>If set to false, the item will be directly put in the player's inventory instead</p>
     */
    DROP_ITEM("dropItem", SettingValueType.BOOLEAN, true, "dropItem"),

    /**
     * The setting for the chance of a reforging to fail
     */
    FAIL_CHANCE("failReforgeChance", SettingValueType.PERCENTAGE, 10, "failReforgeChance"),

    /**
     * The setting for the chance of an additional enchantment being added
     */
    EXTRA_ENCHANTMENT_CHANCE("extraEnchantmentChance", SettingValueType.PERCENTAGE, 5,
            "extraEnchantmentChance"),

    /**
     * The setting for the maximum amount of enchantments that can be added to an item
     */
    MAX_ENCHANTMENTS("maxEnchantments", SettingValueType.POSITIVE_INTEGER, 3, "maxEnchantments"),

    /**
     * The maximum amount of seconds a player may need to wait for the reforging to finish
     */
    MAX_REFORGE_DELAY("delaysInSeconds.maximum", SettingValueType.POSITIVE_INTEGER, 30, "maxReforgeDelay"),

    /**
     * The minimum amount of seconds a player may need to wait for the reforging to finish
     */
    MIN_REFORGE_DELAY("delaysInSeconds.minimum", SettingValueType.POSITIVE_INTEGER, 5, "minReforgeDelay"),

    /**
     * The setting for number of seconds a player has to wait between each usage of the blacksmith
     */
    REFORGE_COOL_DOWN("delaysInSeconds.reforgeCoolDown", SettingValueType.POSITIVE_INTEGER, 60, "reforgeCoolDown"),

    /**
     * The setting for which items the blacksmith is able to reforge
     */
    REFORGE_ABLE_ITEMS("reforgeAbleItems", SettingValueType.STRING_LIST, new String[]{}, "reforgeAbleItems"),

    /**
     * The setting for the title used to display which kind of blacksmith the NPC is
     *
     * <p>While this should be entirely configurable, values such as armor-smith, sword-smith and similar, which
     * describe the blacksmith's specialization, and thus the range of reforge-able items, is expected.</p>
     */
    BLACKSMITH_TITLE("blacksmithTitle", SettingValueType.STRING, "blacksmith", "blacksmithTitle"),

    /*-----------
     | Messages |
     -----------*/

    /**
     * The message displayed when the blacksmith is busy with another player
     */
    BUSY_WITH_PLAYER_MESSAGE("messages.busyPlayerMessage", SettingValueType.STRING,
            "&cI'm busy at the moment. Come back later!", "busyPlayerMessage"),

    /**
     * The message displayed when the blacksmith is already reforging something for the player
     */
    BUSY_WITH_REFORGE_MESSAGE("messages.busyReforgeMessage", SettingValueType.STRING,
            "&cI'm working on it. Be patient! I'll finish {time}!", "busyReforgeMessage"),

    /**
     * The message displayed if the player has to wait for the cool-down to expire
     */
    COOL_DOWN_UNEXPIRED_MESSAGE("messages.coolDownUnexpiredMessage", SettingValueType.STRING,
            "&cYou've already had your chance! Give me a break! I'll be ready {time}!",
            "coolDownUnexpiredMessage"),

    /**
     * The message displayed when displaying the cost of reforging the held item to the player
     */
    COST_MESSAGE("messages.costMessage", SettingValueType.STRING,
            "&eIt will cost &a{cost}&e to reforge that &a{item}&e! Click again to reforge!", "costMessage"),

    /**
     * The message displayed if the blacksmith fails reforging an item
     */
    FAIL_MESSAGE("messages.failReforgeMessage", SettingValueType.STRING,
            "&cWhoops! Didn't mean to do that! Maybe next time?", "failReforgeMessage"),

    /**
     * The message displayed if a player is unable to pay the blacksmith
     */
    INSUFFICIENT_FUNDS_MESSAGE("messages.insufficientFundsMessage", SettingValueType.STRING,
            "&cYou don't have enough money to reforge that item!", "insufficientFundsMessage"),

    /**
     * The message displayed if the blacksmith encounters an item they cannot reforge
     */
    INVALID_ITEM_MESSAGE("messages.invalidItemMessage", SettingValueType.STRING,
            "&cI'm sorry, but I'm a/an {title}, I don't know how to reforge that!", "invalidItemMessage"),

    /**
     * The message displayed if a player presents a different item after seeing the price to reforge an item
     */
    ITEM_UNEXPECTEDLY_CHANGED_MESSAGE("messages.itemChangedMessage", SettingValueType.STRING,
            "&cThat's not the item you wanted to reforge before!", "itemChangedMessage"),

    /**
     * The message displayed when the blacksmith starts reforging an item
     */
    START_REFORGE_MESSAGE("messages.startReforgeMessage", SettingValueType.STRING,
            "&eOk, let's see what I can do...", "startReforgeMessage"),

    /**
     * The message displayed when the blacksmith successfully finishes reforging an item
     */
    SUCCESS_MESSAGE("messages.successMessage", SettingValueType.STRING,
            "There you go! All better!", "successMessage"),

    /**
     * The message displayed when trying to reforge an item with full durability
     */
    NOT_DAMAGED_MESSAGE("messages.notDamagedMessage", SettingValueType.STRING,
            "&cThat item is not in need of repair", "notDamagedMessage");

    private final String path;
    private final String childPath;
    private final Object value;
    private final String commandName;
    private final SettingValueType valueType;

    /**
     * Instantiates a new setting
     *
     * @param path        <p>The full config path for this setting</p>
     * @param valueType   <p>The type of value used by this setting</p>
     * @param value       <p>The default value of this setting</p>
     * @param commandName <p>The name of the command used to change this setting</p>
     */
    NPCSetting(String path, SettingValueType valueType, Object value, String commandName) {
        this.path = "defaults." + path;
        this.value = value;
        this.valueType = valueType;
        this.childPath = path;
        this.commandName = commandName;
    }

    /**
     * Gets the full config path for this setting
     *
     * @return <p>The full config path for this setting</p>
     */
    public String getPath() {
        return path;
    }

    /**
     * Gets the config path without the root node
     *
     * @return <p>The config path without the root node</p>
     */
    public String getChildPath() {
        return childPath;
    }

    /**
     * Gets the value of this setting
     *
     * @return <p>The value of this setting</p>
     */
    public Object getDefaultValue() {
        return value;
    }

    /**
     * The name of the command used to change this setting
     *
     * @return <p>The name of this setting's command</p>
     */
    public String getCommandName() {
        return commandName;
    }

    /**
     * Gets the value type for this setting
     *
     * @return <p>The value type for this setting</p>
     */
    public SettingValueType getValueType() {
        return this.valueType;
    }

}
